# Generated by Django 2.2.8 on 2019-12-08 07:08

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0046_auto_20191207_2357'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_refs',
            name='child',
        ),
        migrations.RemoveField(
            model_name='operation_refs',
            name='parent',
        ),
        migrations.AddField(
            model_name='operation_refs',
            name='income',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, related_name='operation_income', to='planing.Operations'),
        ),
        migrations.AddField(
            model_name='operation_refs',
            name='outcome',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='operation_outcome', to='planing.Operations'),
        ),
    ]
