# Generated by Django 2.2.4 on 2019-08-28 20:56

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0061_auto_20190828_1719'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_resources',
            name='location',
        ),
        migrations.AlterField(
            model_name='operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
    ]
