# Generated by Django 2.2.1 on 2019-05-24 09:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0030_remove_demand_item'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='demand',
            name='customer',
        ),
    ]
