# Generated by Django 2.2.7 on 2019-11-18 14:00

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0106_auto_20191118_1350'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_line',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'),), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='launch_item_refs',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'),), db_index=True, default=1),
        ),
    ]
