# Generated by Django 3.0.2 on 2020-01-20 22:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0070_operations_view'),
    ]

    operations = [
        migrations.CreateModel(
            name='Posting_items',
            fields=[
            ],
            options={
                'verbose_name': 'Оприходования',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('planing.operations',),
        ),
    ]
