# Generated by Django 2.2.8 on 2019-12-04 20:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0148_launch_operation_resources_q_paralel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='q_paralel',
            field=models.PositiveIntegerField(),
        ),
    ]
