# Generated by Django 2.2 on 2019-04-26 10:59

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0013_auto_20190426_1003'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_plan_refs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='child_Operations_plan', to='production.OperationsPlan')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent_Operations_plan', to='production.OperationsPlan')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]
