# Generated by Django 2.2.6 on 2019-10-25 13:11

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0094_auto_20191024_1218'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent_items',
            name='precent_item_type',
        ),
        migrations.AddField(
            model_name='precent',
            name='precent_type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent_item_types'),
        ),
    ]
